@@ -76,23 +76,23 @@ hdr.2d <- function(x, y, prob = c(50, 95, 99), den=NULL, kde.package=c("ash","ks
 #' regions \emph{American Statistician}, \bold{50}, 120-126.
 #' @keywords smooth distribution hplot
 #' @examples
-#'
 #' x <- c(rnorm(200,0,1),rnorm(200,4,1))
 #' y <- c(rnorm(200,0,1),rnorm(200,4,1))
 #' hdr.boxplot.2d(x,y)
 #'
 #' hdrinfo <- hdr.2d(x,y)
 #' plot(hdrinfo, pointcol="red", show.points=TRUE, pch=3)
-#'
 #' @export hdr.boxplot.2d
 hdr.boxplot.2d <- function(x, y, prob=c(50, 99), kde.package=c("ash","ks"), h=NULL,
   xextend=0.15, yextend=0.15, xlab="", ylab="",
-  shadecols=gray((length(prob):1)/(length(prob)+1)), pointcol=1, ...)
+  shadecols="darkgray", pointcol=1,  outside.points=TRUE,...)
 {
+  # Generate shades
+  shadecols  <-  shades(shadecols,length(prob))
   # Estimate bivariate density
   hdr <- hdr.2d(x, y, prob=prob, kde.package=kde.package, h=h, xextend=xextend, yextend=yextend)
   # Produce plot
-  plot(hdr, xlab=xlab, ylab=ylab, shadecols=shadecols, pointcol=pointcol, outside.points=TRUE,...)
+  plot(hdr, xlab=xlab, ylab=ylab, shadecols=shadecols, pointcol=pointcol, outside.points=outside.points,...)
 }
 
 #' @param shaded If \code{TRUE}, the HDR contours are shown as shaded regions.
@@ -104,7 +104,7 @@ hdr.boxplot.2d <- function(x, y, prob=c(50, 99), kde.package=c("ash","ks"), h=NU
 #' @rdname hdr.boxplot.2d
 #' @export
 #' @export plot.hdr2d
-plot.hdr2d <- function(x, shaded=TRUE, show.points=FALSE, outside.points=FALSE, pch=20,  shadecols=gray((length(x$alpha):1)/(length(x$alpha)+1)),
+plot.hdr2d <- function(x, shaded=TRUE, show.points=FALSE, outside.points=FALSE, pch=20, shadecols=gray((length(x$alpha):1)/(length(x$alpha)+1)),
     pointcol=1, ...)
 {
   if(shaded)
@@ -118,7 +118,7 @@ plot.hdr2d <- function(x, shaded=TRUE, show.points=FALSE, outside.points=FALSE,
     index <- (x$fxy < 0.99999*min(x$falpha))
     points(x$x[index], x$y[index], pch=pch, col=pointcol)
   }
-  points(x$mode[1],x$mode[2],pch=19,col="black")
+  points(x$mode[1],x$mode[2],pch=19,col=pointcol)
   invisible(x)
 }
 
@@ -213,3 +213,18 @@ den.estimate.2d <- function(x, y, kde.package=c("ash","ks"), h=NULL, xextend=0.1
   return(den)
 }
 
+
+
+#' Alpha
+#'
+#' A simple function to change the opacity of a color
+#' @param  color the name or idea of a R color
+#' @param  alpha a value in [0,1] defining the opacity wanted.
+alpha <- function(color,alpha) rgb(t(col2rgb(color)/255),alpha=alpha)
+
+#' Shades
+#'
+#' A simple function to genarte shade of one color by changing its opacity
+#' @param  color the name or idea of a R color
+#' @param  n number or shades wanted
+shades<-function(color,n) sapply(seq(0,1,length.out=n+1),alpha,color=color)[2:(n+1)]
